@@ -65,7 +65,6 @@ func determineFieldOrder() []string {
 		"Configurations",
 		"Generators",
 		"Transformers",
-		"Inventory",
 		"Components",
 		"OpenAPI",
 		"BuildMetadata",
@@ -165,21 +164,18 @@ func (mf *kustomizationFile) Read() (*types.Kustomization, error) {
 	if err != nil {
 		return nil, err
 	}
-	data, err = types.FixKustomizationPreUnmarshalling(data)
-	if err != nil {
-		return nil, err
-	}
+
 	var k types.Kustomization
-	err = k.Unmarshal(data)
-	if err != nil {
+	if err := k.Unmarshal(data); err != nil {
 		return nil, err
 	}
-	k.FixKustomizationPostUnmarshalling()
-	err = mf.parseCommentedFields(data)
-	if err != nil {
+
+	k.FixKustomization()
+
+	if err := mf.parseCommentedFields(data); err != nil {
 		return nil, err
 	}
-	return &k, err
+	return &k, nil
 }
 
 func (mf *kustomizationFile) Write(kustomization *types.Kustomization) error {
@@ -264,12 +260,13 @@ func (mf *kustomizationFile) hasField(name string) bool {
 }
 
 /*
- isCommentOrBlankLine determines if a line is a comment or blank line
- Return true for following lines
- # This line is a comment
-       # This line is also a comment with several leading white spaces
+isCommentOrBlankLine determines if a line is a comment or blank line
+Return true for following lines
+# This line is a comment
 
- (The line above is a blank line)
+	# This line is also a comment with several leading white spaces
+
+(The line above is a blank line)
 */
 func isCommentOrBlankLine(line []byte) bool {
 	s := bytes.TrimRight(bytes.TrimLeft(line, " "), "\n")
